unit NetInfo;

interface

uses Winsock, SysUtils, Windows, NB30;

function GetLocalIP(var IP: string): Boolean;
function SetComputerName(name: string): boolean;
function GetComputerName: string;
function GetMACAddress: string;
function GetUserFromWindows: string;

implementation

//-----------------------------------------------------------------------------
// Purpose:   Get local Windows user name
//-----------------------------------------------------------------------------

function GetUserFromWindows: string;
var
  UserName: string;
  UserNameLen: Dword;
begin
  UserNameLen := 255;
  SetLength(userName, UserNameLen);
  if GetUserName(PChar(UserName), UserNameLen) then
    Result := Copy(UserName, 1, UserNameLen - 1)
  else
    Result := 'Unknown';
end;

//-----------------------------------------------------------------------------
// Purpose:   Get local IP settings.
//-----------------------------------------------------------------------------

function GetLocalIP(var IP: string): Boolean;
var
  WSData: TWSAData;
  HostName: array[0..255] of char;
  HostEnt: PHostEnt;
  Addr: PChar;
begin
  Result := False;
  WSAStartup($0101, WSData);
  try
    GetHostName(HostName, SizeOf(HostName));
    HostEnt := GetHostByName(HostName);
    if Assigned(HostEnt) and Assigned(HostEnt^.h_addr_list) then
    begin
      Addr := HostEnt^.h_addr_list^;
      if Assigned(Addr) then
      begin
        IP := Format('%d.%d.%d.%d', [Byte(Addr[0]),
          Byte(Addr[1]), Byte(addr[2]), Byte(addr[3])]);
        Result := True;
      end
    end;
  finally
    WSACleanup;
  end
end;

//-----------------------------------------------------------------------------
// Purpose:   Set workstation name
//-----------------------------------------------------------------------------

function SetComputerName(name: string): boolean;
var
  ComputerName: array[0..MAX_COMPUTERNAME_LENGTH + 1] of Char;
begin
  StrPCopy(ComputerName, name);
  Result := Windows.SetComputerName(ComputerName);
end;

//-----------------------------------------------------------------------------
// Purpose:   Get workstation name
//-----------------------------------------------------------------------------

function GetComputerName: string;
var
  buffer: array[0..MAX_COMPUTERNAME_LENGTH + 1] of Char;
  Size: Cardinal;
begin
  Size := MAX_COMPUTERNAME_LENGTH + 1;
  Windows.GetComputerName(@buffer, Size);
  Result := StrPas(buffer);
end;

//-----------------------------------------------------------------------------
// Purpose:   Get NetBios info from adapter
//-----------------------------------------------------------------------------

function GetAdapterInfo(Lana: Char): string;
var
  Adapter: TAdapterStatus;
  NCB: TNCB;
begin
  FillChar(NCB, SizeOf(NCB), 0);
  NCB.ncb_command := Char(NCBRESET);
  NCB.ncb_lana_num := Lana;
  if Netbios(@NCB) <> Char(NRC_GOODRET) then
  begin
    Result := 'NetBios info not found.';
    Exit;
  end;

  FillChar(NCB, SizeOf(NCB), 0);
  NCB.ncb_command := Char(NCBASTAT);
  NCB.ncb_lana_num := Lana;
  NCB.ncb_callname := '*';

  FillChar(Adapter, SizeOf(Adapter), 0);
  NCB.ncb_buffer := @Adapter;
  NCB.ncb_length := SizeOf(Adapter);
  if Netbios(@NCB) <> Char(NRC_GOODRET) then
  begin
    Result := 'NetBios info not found.';
    Exit;
  end
  else
  begin
    Result :=
      IntToHex(Byte(Adapter.adapter_address[0]), 2) + '-' +
      IntToHex(Byte(Adapter.adapter_address[1]), 2) + '-' +
      IntToHex(Byte(Adapter.adapter_address[2]), 2) + '-' +
      IntToHex(Byte(Adapter.adapter_address[3]), 2) + '-' +
      IntToHex(Byte(Adapter.adapter_address[4]), 2) + '-' +
      IntToHex(Byte(Adapter.adapter_address[5]), 2);
  end;
end;

//-----------------------------------------------------------------------------
// Procedure: GetMACAdress
// Purpose:   Get local MAC address
//-----------------------------------------------------------------------------

function GetMACAddress: string;
var
  AdapterList: TLanaEnum;
  NCB: TNCB;
begin
  FillChar(NCB, SizeOf(NCB), 0);
  NCB.ncb_command := Char(NCBENUM);
  NCB.ncb_buffer := @AdapterList;
  NCB.ncb_length := SizeOf(AdapterList);
  Netbios(@NCB);
  if Byte(AdapterList.length) > 0 then
    Result := GetAdapterInfo(AdapterList.lana[0])
  else
    Result := 'MAC not found.';
end;

end.

